#!/usr/bin/env python3
from typing import Any, Dict, Optional

from . import get_module_logger

logger = get_module_logger(__name__)


def dictdiff(s1: Optional[Dict[Any, Any]], s2: Optional[Dict[Any, Any]]) -> Optional[Dict[Any, Any]]:
    if not s2 or s1 == s2:
        return None
    if not s1:
        return s2  # s1 is empty.

    dst = dict()
    for k in s1.keys() | s2.keys():
        v1 = s1.get(k)
        v2 = s2.get(k)
        if v1 == v2:
            continue  # Not changed
        if v1 is None:
            logger.debug(f"Added Item ({k})  : None -> {v2}")
            dst[k] = v2  # Added Item
            continue
        if v2 is None:
            logger.debug(f"Removed Item ({k}): {v1} -> None")
            dst[k] = None  # Removed Item
            continue
        if type(v1) != type(v2):
            dst[k] = v2  # Type changed. Override.
            continue

        # here, v1 != v2 and type(v1) == type(v2)
        if isinstance(v1, dict):
            dst[k] = dictdiff(v1, v2)
        else:
            logger.debug(f"Updated Item ({k}): {v1} -> {v2}")
            dst[k] = v2  # Updated item

    return dst



from concurrent.futures import Future
from os.path import expanduser
from typing import Any, Dict, Optional
from uuid import uuid4

from awscrt import auth, exceptions, http, io, mqtt
from awsiot import mqtt_connection_builder

from . import ExceptionAwsIotClient, get_module_logger

logger = get_module_logger(__name__)
KwArgs = Optional[Dict[str, Any]]


class ExceptionAwsIotMqtt(ExceptionAwsIotClient):
    def __init__(self, *args: object) -> None:
        super().__init__(*args)


def on_connection_interrupted(connection: mqtt.Connection, error: exceptions.AwsCrtError, **kwargs: KwArgs) -> None:
    logger.debug(f"Connection interrupted. error: {error}")


def on_connection_resumed(
    connection: mqtt.Connection, return_code: mqtt.ConnectReturnCode, session_present: bool, **kwargs: KwArgs
) -> None:
    logger.debug(f"Connection resumed. return_code: {return_code} session_present: {session_present}")

    if return_code == mqtt.ConnectReturnCode.ACCEPTED and not session_present:
        logger.debug("Session did not persist. Resubscribing to existing topics...")
        resubscribe_future, _ = connection.resubscribe_existing_topics()

        # Cannot synchronously wait for resubscribe result
        # because we're on the connection's event-loop thread,
        # evaluate result with a callback instead.
        resubscribe_future.add_done_callback(on_resubscribe_complete)


def on_resubscribe_complete(resubscribe_future: Future) -> None:  # type: ignore
    resubscribe_results = resubscribe_future.result()
    logger.debug(f"Resubscribe results: {resubscribe_results}")

    for topic, qos in resubscribe_results["topics"]:
        if qos is None:
            raise (ExceptionAwsIotMqtt(f"Server rejected resubscribe to topic: {topic}"))


class ConnectionParams:
    def __init__(
        self,
        endpoint: str = "",
        signing_region: str = "ap-northeast-1",
        root_ca: str = "~/.aws/cert/AmazonRootCA1.pem",
        cert: str = "~/.aws/cert/certificate.pem.crt",
        key: str = "~/.aws/cert/private.pem.key",
        client_id: str = "mqtt-" + str(uuid4()),
        use_websocket: bool = False,
        proxy_host: Optional[str] = None,
        proxy_port: int = 8080,
    ) -> None:
        self.endpoint = endpoint
        self.signing_region = signing_region
        self.root_ca = expanduser(root_ca)
        self.cert = expanduser(cert)
        self.key = expanduser(key)
        self.client_id = client_id
        self.use_websocket = use_websocket
        self.proxy_host = proxy_host
        self.proxy_port = proxy_port


def init(params: ConnectionParams) -> mqtt.Connection:
    # Spin up resources
    event_loop_group = io.EventLoopGroup(1)
    host_resolver = io.DefaultHostResolver(event_loop_group)
    client_bootstrap = io.ClientBootstrap(event_loop_group, host_resolver)

    if params.use_websocket:
        proxy_options = None
        if params.proxy_host:
            proxy_options = http.HttpProxyOptions(host_name=params.proxy_host, port=params.proxy_port)

        credentials_provider = auth.AwsCredentialsProvider.new_default_chain(client_bootstrap=client_bootstrap)
        mqtt_connection = mqtt_connection_builder.websockets_with_default_aws_signing(
            endpoint=params.endpoint,
            client_bootstrap=client_bootstrap,
            region=params.signing_region,
            credentials_provider=credentials_provider,
            websocket_proxy_options=proxy_options,
            ca_filepath=params.root_ca,
            on_connection_interrupted=on_connection_interrupted,
            on_connection_resumed=on_connection_resumed,
            clent_id=params.client_id,
            clean_session=False,
            keep_alive_secs=6,
        )

    else:
        mqtt_connection = mqtt_connection_builder.mtls_from_path(
            endpoint=params.endpoint,
            cert_filepath=params.cert,
            pri_key_filepath=params.key,
            client_bootstrap=client_bootstrap,
            ca_filepath=params.root_ca,
            on_connection_interrupted=on_connection_interrupted,
            on_connection_resumed=on_connection_resumed,
            client_id=params.client_id,
            clean_session=False,
            keep_alive_secs=6,
        )

    logger.debug("Connecting to {} with client ID '{}'...".format(params.endpoint, params.client_id))

    return mqtt_connection



import threading
from concurrent.futures import Future
from typing import Any, Callable, Dict, Optional

from awscrt import mqtt
from awsiot import iotjobs

from . import ExceptionAwsIotClient, get_module_logger

logger = get_module_logger(__name__)
JobDocument = Optional[Dict[str, Any]]


class ExceptionAwsIotJobs(ExceptionAwsIotClient):
    def __init__(self, *args: object) -> None:
        super().__init__(*args)


class ExceptionAwsIotJobsUserDefinedFailure(ExceptionAwsIotJobs):
    def __init__(self, *args: object) -> None:
        super().__init__(*args)


def empty_func(id: str, document: JobDocument) -> None:
    logger.debug(f"empty_job. id: {id}, document: {document}")


class LockedData:
    def __init__(self) -> None:
        self.lock: threading.Lock = threading.Lock()
        self.disconnect_called: bool = False
        self.is_working_on_job: bool = False
        self.is_next_job_waiting: bool = False


class client:
    def __init__(
        self,
        connection: mqtt.Connection,
        thing_name: str,
        qos: mqtt.QoS = mqtt.QoS.AT_LEAST_ONCE,
        job_func: Callable[[str, JobDocument], None] = empty_func,
    ) -> None:
        self.client = iotjobs.IotJobsClient(connection)
        self.thing_name = thing_name
        self.qos = qos
        self.locked_data = LockedData()
        self.job_func = job_func

        try:
            changed_subscription_request = iotjobs.NextJobExecutionChangedSubscriptionRequest(thing_name=thing_name)
            (subscribed_future, _,) = self.client.subscribe_to_next_job_execution_changed_events(
                request=changed_subscription_request,
                qos=qos,
                callback=self.on_next_job_execution_changed,
            )

            # Wait for subscription to succeed
            subscribed_future.result()

            logger.debug("Subscribing to Start responses...")
            start_subscription_request = iotjobs.StartNextPendingJobExecutionSubscriptionRequest(thing_name=thing_name)
            (subscribed_accepted_future, _,) = self.client.subscribe_to_start_next_pending_job_execution_accepted(
                request=start_subscription_request,
                qos=qos,
                callback=self.on_start_next_pending_job_execution_accepted,
            )

            (subscribed_rejected_future, _,) = self.client.subscribe_to_start_next_pending_job_execution_rejected(
                request=start_subscription_request,
                qos=qos,
                callback=self.on_start_next_pending_job_execution_rejected,
            )

            # Wait for subscriptions to succeed
            subscribed_accepted_future.result()
            subscribed_rejected_future.result()

            logger.debug("Subscribing to Update responses...")
            # Note that we subscribe to "+", the MQTT wildcard, to receive
            # responses about any job-ID.
            update_subscription_request = iotjobs.UpdateJobExecutionSubscriptionRequest(thing_name=thing_name, job_id="+")

            (subscribed_accepted_future, _,) = self.client.subscribe_to_update_job_execution_accepted(
                request=update_subscription_request,
                qos=qos,
                callback=self.on_update_job_execution_accepted,
            )

            (subscribed_rejected_future, _,) = self.client.subscribe_to_update_job_execution_rejected(
                request=update_subscription_request,
                qos=qos,
                callback=self.on_update_job_execution_rejected,
            )

            # Wait for subscriptions to succeed
            subscribed_accepted_future.result()
            subscribed_rejected_future.result()

            # Make initial attempt to start next job. The service should reply with
            # an "accepted" response, even if no jobs are pending. The response
            # will contain data about the next job, if there is one.
            self.try_start_next_job()

        except Exception as e:
            raise (e)

    def try_start_next_job(self) -> None:
        logger.debug("Trying to start the next job...")
        with self.locked_data.lock:
            if self.locked_data.is_working_on_job:
                logger.debug("Nevermind, already working on a job.")
                return

            if self.locked_data.disconnect_called:
                logger.debug("Nevermind, sample is disconnecting.")
                return

            self.locked_data.is_working_on_job = True
            self.locked_data.is_next_job_waiting = False

        logger.debug("Publishing request to start next job...")
        request = iotjobs.StartNextPendingJobExecutionRequest(thing_name=self.thing_name)
        publish_future = self.client.publish_start_next_pending_job_execution(request, self.qos)
        publish_future.add_done_callback(self.on_publish_start_next_pending_job_execution)

    def done_working_on_job(self) -> None:
        with self.locked_data.lock:
            self.locked_data.is_working_on_job = False
            try_again = self.locked_data.is_next_job_waiting

        if try_again:
            self.try_start_next_job()

    def on_next_job_execution_changed(self, event: iotjobs.NextJobExecutionChangedEvent) -> None:
        try:
            execution = event.execution
            if execution:
                logger.debug(
                    "Received Next Job Execution Changed event. ",
                    "job_id:{} job_document:{}".format(execution.job_id, execution.job_document),
                )

                # Start job now, or remember to start it when current job is done
                start_job_now = False
                with self.locked_data.lock:
                    if self.locked_data.is_working_on_job:
                        self.locked_data.is_next_job_waiting = True
                    else:
                        start_job_now = True

                if start_job_now:
                    self.try_start_next_job()

            else:
                logger.debug("Received Next Job Execution Changed event: ", "None. Waiting for further jobs...")

        except Exception as e:
            raise (e)

    def on_publish_start_next_pending_job_execution(self, future: Future) -> None:  # type: ignore
        try:
            future.result()  # raises exception if publish failed

            logger.debug("Published request to start the next job.")

        except Exception as e:
            raise (e)

    def on_start_next_pending_job_execution_accepted(self, response):
        # type: (iotjobs.StartNextJobExecutionResponse) -> None
        try:
            if response.execution:
                execution = response.execution
                logger.debug(
                    "Request to start next job was accepted. job_id:{} job_document:{}".format(
                        execution.job_id, execution.job_document
                    )
                )

                # To emulate working on a job, spawn a thread that sleeps for a few seconds
                job_thread = threading.Thread(
                    target=lambda: self.job_thread_fn(execution.job_id, execution.job_document),
                    name="job_thread",
                )
                job_thread.start()
            else:
                logger.debug(
                    "Request to start next job was accepted, but there are no jobs to be done.", " Waiting for further jobs..."
                )
                self.done_working_on_job()

        except Exception as e:
            raise (e)

    def on_start_next_pending_job_execution_rejected(self, rejected: iotjobs.RejectedError) -> None:
        raise ExceptionAwsIotJobs(
            "Request to start next pending job rejected with code:'{}' message:'{}'".format(rejected.code, rejected.message)
        )

    def job_thread_fn(self, job_id: str, job_document: JobDocument) -> None:
        try:
            logger.debug("Starting local work on job...")
            self.job_func(job_id, job_document)
            logger.debug("Done working on job.")
        except ExceptionAwsIotJobsUserDefinedFailure as e:
            logger.debug("Report that job excecution failed due to user-defined reason...")
            request = iotjobs.UpdateJobExecutionRequest(
                thing_name=self.thing_name,
                job_id=job_id,
                status_details=dict(failure_type="user_defined", failure_detail=str(e)),
                status=iotjobs.JobStatus.FAILED,
            )
        except Exception as e:
            logger.debug("Report that job excecution failed due to unknown reason...")
            request = iotjobs.UpdateJobExecutionRequest(
                thing_name=self.thing_name,
                job_id=job_id,
                status_details=dict(failure_type="unknown", failure_detail=str(e)),
                status=iotjobs.JobStatus.FAILED,
            )
        else:
            logger.debug("Publishing request to update job status to SUCCEEDED...")
            request = iotjobs.UpdateJobExecutionRequest(
                thing_name=self.thing_name,
                job_id=job_id,
                status=iotjobs.JobStatus.SUCCEEDED,
            )
        finally:
            publish_future = self.client.publish_update_job_execution(request, self.qos)
            publish_future.add_done_callback(self.on_publish_update_job_execution)

    def on_publish_update_job_execution(self, future: Future) -> None:  # type: ignore
        try:
            future.result()  # raises exception if publish failed
            logger.debug("Published request to update job.")

        except Exception as e:
            raise (e)

    def on_update_job_execution_accepted(self, response: iotjobs.UpdateJobExecutionResponse) -> None:
        try:
            logger.debug("Request to update job was accepted.")
            self.done_working_on_job()
        except Exception as e:
            raise (e)

    def on_update_job_execution_rejected(self, rejected: iotjobs.RejectedError) -> None:
        raise ExceptionAwsIotJobs(
            "Request to update job status was rejected. code:'{}' message:'{}'.".format(rejected.code, rejected.message)
        )



import logging


class ExceptionAwsIotClient(Exception):
    def __init__(self, *args: object) -> None:
        super().__init__(*args)


def get_module_logger(modname: str) -> logging.Logger:
    logger = logging.getLogger(modname)
    handler = logging.NullHandler()
    logger.addHandler(handler)
    return logger


__all__ = [
    "mqtt",
    "pubsub",
    "classic_shadow",
    "named_shadow",
    "jobs",
    "get_module_logger",
]



import threading
from concurrent.futures import Future
from traceback import format_exc
from typing import Any, Callable, Dict, Optional

from awscrt import mqtt
from awsiot import iotshadow

from . import ExceptionAwsIotClient, dictdiff, get_module_logger

logger = get_module_logger(__name__)
ShadowDocument = Optional[Dict[str, Any]]


class ExceptionAwsIotClassicShadow(ExceptionAwsIotClient):
    def __init__(self, *args: object) -> None:
        super().__init__(*args)


class ExceptionAwsIotClassicShadowInvalidDelta(ExceptionAwsIotClassicShadow):
    def __init__(self, *args: object) -> None:
        super().__init__(*args)


def empty_func(thing_name: str, shadow_property: str, value: ShadowDocument) -> None:
    logger.debug(f"empty func. thing_name: {thing_name}, property: {shadow_property}, value: {value}")


SHADOW_VALUE_DEFAULT = None


class LockedData:
    def __init__(self) -> None:
        self.lock = threading.Lock()
        self.shadow_value: ShadowDocument = None
        self.desired_value: ShadowDocument = None
        self.disconnect_called: bool = False


class client:
    def __init__(
        self,
        connection: mqtt.Connection,
        thing_name: str,
        shadow_property: str,
        qos: mqtt.QoS = mqtt.QoS.AT_LEAST_ONCE,
        delta_func: Callable[[str, str, ShadowDocument], None] = empty_func,
        publish_full_doc: bool = False,
    ) -> None:
        self.client = iotshadow.IotShadowClient(connection)
        self.thing_name = thing_name
        self.shadow_property = shadow_property
        self.locked_data = LockedData()
        self.delta_func = delta_func
        self.qos = qos
        self.publish_full_doc = publish_full_doc
        try:
            # Subscribe to necessary topics.
            # Note that **is** important to wait for "accepted/rejected" subscriptions
            # to succeed before publishing the corresponding "request".
            logger.debug("Subscribing to Delta events...")
            (delta_subscribed_future, _,) = self.client.subscribe_to_shadow_delta_updated_events(
                request=iotshadow.ShadowDeltaUpdatedSubscriptionRequest(thing_name=thing_name),
                qos=qos,
                callback=self.on_shadow_delta_updated,
            )

            # Wait for subscription to succeed
            delta_subscribed_future.result()

            logger.debug("Subscribing to Update responses...")
            (update_accepted_subscribed_future, _,) = self.client.subscribe_to_update_shadow_accepted(
                request=iotshadow.UpdateShadowSubscriptionRequest(thing_name=thing_name),
                qos=qos,
                callback=self.on_update_shadow_accepted,
            )

            (update_rejected_subscribed_future, _,) = self.client.subscribe_to_update_shadow_rejected(
                request=iotshadow.UpdateShadowSubscriptionRequest(thing_name=thing_name),
                qos=qos,
                callback=self.on_update_shadow_rejected,
            )

            # Wait for subscriptions to succeed
            update_accepted_subscribed_future.result()
            update_rejected_subscribed_future.result()

            logger.debug("Subscribing to Get responses...")
            (get_accepted_subscribed_future, _,) = self.client.subscribe_to_get_shadow_accepted(
                request=iotshadow.GetShadowSubscriptionRequest(thing_name=thing_name),
                qos=qos,
                callback=self.on_get_shadow_accepted,
            )

            (get_rejected_subscribed_future, _,) = self.client.subscribe_to_get_shadow_rejected(
                request=iotshadow.GetShadowSubscriptionRequest(thing_name=thing_name),
                qos=qos,
                callback=self.on_get_shadow_rejected,
            )

            # Wait for subscriptions to succeed
            get_accepted_subscribed_future.result()
            get_rejected_subscribed_future.result()

            # The rest of the sample runs asyncronously.

            # Issue request for shadow's current state.
            # The response will be received by the on_get_accepted() callback
            logger.debug("Requesting current shadow state...")
            publish_get_future = self.client.publish_get_shadow(
                request=iotshadow.GetShadowRequest(thing_name=thing_name), qos=qos
            )

            # Ensure that publish succeeds
            publish_get_future.result()

        except Exception as e:
            logger.error(format_exc())
            raise (e)

    def on_get_shadow_accepted(self, response):
        # type: (iotshadow.GetShadowResponse) -> None
        try:
            logger.debug("Finished getting initial shadow state.")

            with self.locked_data.lock:
                if self.locked_data.shadow_value is not None:
                    logger.debug("  Ignoring initial query because a delta event has already been received.")
                    return

            if response.state:
                if response.state.delta:
                    value = response.state.delta.get(self.shadow_property)
                    if value:
                        logger.debug("  Shadow contains delta value '{}'.".format(value))
                        return

                if response.state.reported:
                    value = response.state.reported.get(self.shadow_property)
                    if value:
                        logger.debug("  Shadow contains reported value '{}'.".format(value))
                        self.set_local_value_due_to_initial_query(response.state.reported[self.shadow_property])
                        return

            logger.debug("  Shadow document lacks '{}' property. Setting defaults...".format(self.shadow_property))
            self.change_reported_value(SHADOW_VALUE_DEFAULT)
            return

        except Exception as e:
            logger.error(format_exc())
            raise (e)

    def on_get_shadow_rejected(self, error):
        # type: (iotshadow.ErrorResponse) -> None
        if error.code == 404:
            logger.debug("Thing has no shadow document. Creating with defaults...")
            self.change_reported_value(SHADOW_VALUE_DEFAULT)
        else:
            raise ExceptionAwsIotClassicShadow(
                "Get request was rejected. code:{} message:'{}'".format(error.code, error.message)
            )

    def on_shadow_delta_updated(self, delta):
        # type: (iotshadow.ShadowDeltaUpdatedEvent) -> None
        try:
            logger.debug("Received shadow delta event.")
            if delta.state and (self.shadow_property in delta.state):
                value = delta.state[self.shadow_property]
                if value is None:
                    logger.debug("  Delta reports that '{}' was deleted. Resetting defaults...".format(self.shadow_property))
                    self.change_reported_value(SHADOW_VALUE_DEFAULT)
                    return
                else:
                    logger.debug("  Delta reports that desired value is '{}'. Invoke delta func...".format(value))
                    try:
                        self.delta_func(self.thing_name, self.shadow_property, value)
                    except ExceptionAwsIotClassicShadowInvalidDelta:
                        logger.debug(
                            "  Delta reports invalid request in {}. Resetting defaults...".format(self.shadow_property)
                        )
                        self.change_desired_value(SHADOW_VALUE_DEFAULT)
            else:
                logger.debug("  Delta did not report a change in '{}'".format(self.shadow_property))

        except Exception as e:
            logger.error(format_exc())
            raise (e)

    def on_publish_update_shadow(self, future: Future) -> None:  # type: ignore
        try:
            future.result()
            logger.debug("Update request published.")
        except Exception as e:
            logger.error(format_exc())
            logger.debug("Failed to publish update request.")
            raise (e)

    def on_update_shadow_accepted(self, response):
        # type: (iotshadow.UpdateShadowResponse) -> None
        try:
            if response.state.reported:
                logger.debug("Finished updating reported shadow value to '{}'.".format(response.state.reported))
            if response.state.desired:
                logger.debug("Finished updating desired shadow value to '{}'.".format(response.state.desired))
        except Exception as e:
            logger.error(format_exc())
            logger.error("Updated shadow is missing the target property.")
            raise (e)

    def on_update_shadow_rejected(self, error):
        # type: (iotshadow.ErrorResponse) -> None
        errstr = "Update request was rejected. code:{} message:'{}'".format(error.code, error.message)
        logger.error(errstr)
        raise ExceptionAwsIotClient(errstr)

    def set_local_value_due_to_initial_query(self, reported_value: ShadowDocument) -> None:
        with self.locked_data.lock:
            self.locked_data.shadow_value = reported_value

    def change_reported_value(self, value: ShadowDocument) -> None:
        with self.locked_data.lock:
            if self.locked_data.shadow_value == value:
                logger.debug("Local value is already '{}'.".format(value))
                return

            if self.publish_full_doc:
                reported = value
            else:
                reported = dictdiff.dictdiff(self.locked_data.shadow_value, value)

            logger.debug("Changed local shadow value to '{}'.".format(value))
            self.locked_data.shadow_value = value

        logger.debug("Updating reported shadow value to '{}'...".format(reported))
        request = iotshadow.UpdateShadowRequest(
            thing_name=self.thing_name,
            state=iotshadow.ShadowState(
                reported={self.shadow_property: reported},
            ),
        )
        future = self.client.publish_update_shadow(request, self.qos)
        future.add_done_callback(self.on_publish_update_shadow)

    def change_desired_value(self, value: ShadowDocument) -> None:
        with self.locked_data.lock:
            if self.locked_data.desired_value == value:
                logger.debug("Local desired value is already '{}'.".format(value))
                return

            if self.publish_full_doc:
                desired = value
            else:
                desired = dictdiff.dictdiff(self.locked_data.desired_value, value)

            logger.debug("Changed local desired value to '{}'.".format(value))
            self.locked_data.desired_value = value

        logger.debug("Updating desired shadow value to '{}'...".format(desired))
        request = iotshadow.UpdateShadowRequest(
            thing_name=self.thing_name,
            state=iotshadow.ShadowState(
                desired={self.shadow_property: desired},
            ),
        )
        future = self.client.publish_update_shadow(request, self.qos)
        future.add_done_callback(self.on_publish_update_shadow)



import json
from typing import Any, Callable, Dict, Optional

from awscrt import mqtt
from awsiotclient import get_module_logger

logger = get_module_logger(__name__)


def empty_func(topic: str, payload: Optional[Dict[str, Any]], **kwargs: Optional[Dict[str, Any]]) -> None:
    logger.debug(f"empty_func: {topic}, {payload}")


class Subscriber:
    def __init__(
        self,
        connection: mqtt.Connection,
        topic: str,
        qos: mqtt.QoS = mqtt.QoS.AT_LEAST_ONCE,
        callback: Callable[[str, Optional[Dict[str, Any]]], None] = empty_func,
    ):
        self.callback = callback
        self.future, self.packet_id = connection.subscribe(topic=topic, qos=qos, callback=self.on_message_received)

        self.result = self.future.result()
        logger.debug(f"Subscribed with {str(self.result['qos'])}")

    def on_message_received(self, topic: str, payload: bytes) -> None:
        packet = json.loads(payload)
        self.callback(topic, packet)
        logger.debug(f"Received message from topic '{topic}': {packet}")


class Publisher:
    def __init__(self, connection: mqtt.Connection, topic: str, qos: mqtt.QoS = mqtt.QoS.AT_LEAST_ONCE):
        self.connection = connection
        self.topic = topic
        self.qos = qos

    def publish(self, payload: Optional[Dict[str, Any]]) -> None:
        if payload:
            logger.debug(f"Publishing message to topic '{self.topic}': {payload}")
            self.connection.publish(topic=self.topic, payload=json.dumps(payload), qos=self.qos)
        else:
            logger.debug(f"Empty payload. Nothing will be publshed to '{self.topic}'")



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['awsiotclient']

package_data = \
{'': ['*']}

install_requires = \
['awsiotsdk>=1.5.11,<2.0.0']

setup_kwargs = {
    'name': 'awsiotclient',
    'version': '0.1.1',
    'description': 'The AWS IoT Device Client provides device-side functionality for AWS IoT services such as jobs, device shadow, and simple pubsub.',
    'long_description': '# aws-iot-device-client-python\n\nThe AWS IoT Device Client provides device-side functionality for AWS IoT services such as jobs, device shadow, and simple pubsub.\n\n## Installation\n\n### Minimum Requirements\n\n- Python 3.6+\n\n### Install from PyPI\n\n```shell\npython3 -m pip install awsiotclient\n```\n\n### Install from source\n\n```shell\ngit clone https://github.com/whill-labs/aws-iot-device-client-python\npython3 -m pip install ./aws-iot-device-client-python\n```\n\n## Usage\n\n### Create MQTT connection\n\n```python\nfrom awsiotclient import mqtt\n\n# Construct connection parameters\nconn_params = mqtt.ConnectionParams()\n\n# Required params\nconn_params.endpoint = <your_endpoint_url>\nconn_params.cert = <path_to_your_certificate>\nconn_params.key = <path_to_your_private_key>\nconn_params.root_ca = <path_to_your_root_ca>\n\n# Optional params\nconn_params.client_id = <client_id> # default "mqtt-" + uuid4()\nconn_params.signing_region = <signing_region> # default "ap-northeast-1" (Tokyo Region)\nconn_params.use_websocket = <True/False> # default False\n\n# Initialize connection\nmqtt_connection = mqtt.init(conn_params)\nconnect_future = mqtt_connection.connect()\nconnect_future.result()\n```\n\n### Use AWS IoT named shadow\n\nNote that usage of classic shadow client is similar to named shadow client.\n\n#### without delta (one-way communication from device to cloud)\n\n```python\nfrom awsiotclient import mqtt, named_shadow\n\n# <create mqtt connection here as described above>\nmy_client = named_shadow.client(\n    mqtt_connection,\n    thing_name="my_thing",\n    shadow_name="my_shadow"\n)\n\nmy_value = dict(foo="var")\nmy_clinet.change_reported_value(my_value)\n```\n\n#### with delta (two-way communication from/to device and cloud)\n\n```python\nfrom awsiotclient import mqtt, named_shadow\n\ndef my_delta_func(thing_name: str, shadow_name: str, value: Dict[str, Any]) -> None:\n    print("my_client invokes this callback when it receives delta message")\n    print(f"thing_name: {thing_name}, shadow_name: {shadow_name}, value: {value}")\n\n# <create mqtt connection here as described above>\nmy_client = named_shadow.client(\n    mqtt_connection,\n    thing_name="my_thing",\n    shadow_name="my_shadow",\n    delta_func=my_delta_func,\n)\n\nmy_value = dict(foo="var")\nmy_client.change_reported_value(my_value)\n# <wait until the client receives delta>\n```\n\n### Use AWS IoT jobs\n\n```python\nfrom awsiotclient import mqtt, named_shadow\n\ndef job_runner(id: str, document: dict):\n    print("my_client invokes this callback when it receives job document")\n    print(f"job id: {id}, document: {document}")\n\n# <create mqtt connection here as described above\njob_client = jobs.client(\n    mqtt_connection,\n    thing_name="my_thing",\n    job_func=job_runner\n)\n# <wait until the client receives job>\n```\n\n## License\n\nThis library is licensed under the Apache 2.0 License.\n\n## Acknowledgments\n\n- [AWS IoT Device SDK v2 for Python](https://github.com/aws/aws-iot-device-sdk-python-v2)\n',
    'author': 'Seiya Shimizu',
    'author_email': 'seiya.shimizu@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/whill-labs/aws-iot-device-client-python',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
